# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['deathstar',
 'deathstar.crystals.lateral_movement',
 'deathstar.crystals.postex',
 'deathstar.crystals.privesc.domain',
 'deathstar.crystals.privesc.local',
 'deathstar.crystals.recon']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.16.0,<0.17.0', 'rich>=9.4.0,<10.0.0']

entry_points = \
{'console_scripts': ['deathstar = deathstar.deathstar:run']}

setup_kwargs = {
    'name': 'deathstar-empire',
    'version': '0.2.0',
    'description': "Uses Empire's RESTful API to automate gaining Domain and/or Enterprise Admin rights in Active Directory environments using some of the most common offensive TTPs",
    'long_description': '# DeathStar\n\n<p align="center">\n  <img src="https://user-images.githubusercontent.com/5151193/88892241-ddc6d700-d21a-11ea-9c37-3cffed86e2f8.png" alt="DeathStar" height="300"/>\n</p>\n\nDeathStar is a Python script that uses [Empire\'s](https://github.com/BC-SECURITY/Empire) RESTful API to automate gaining Domain and/or Enterprise Admin rights in Active Directory environments using some of the most common offensive TTPs.\n\n# Sponsors\n[<img src="https://www.blackhillsinfosec.com/wp-content/uploads/2016/03/BHIS-logo-L-300x300.png" width="130" height="130"/>](https://www.blackhillsinfosec.com/)\n[<img src="https://handbook.volkis.com.au/assets/img/Volkis_Logo_Brandpack.svg" width="130" hspace="10"/>](https://volkis.com.au)\n[<img src="https://user-images.githubusercontent.com/5151193/85817125-875e0880-b743-11ea-83e9-764cd55a29c5.png" width="200" vspace="21"/>](https://qomplx.com/blog/cyber/)\n[<img src="https://user-images.githubusercontent.com/5151193/86521020-9f0f4e00-be21-11ea-9256-836bc28e9d14.png" width="250" hspace="20"/>](https://ledgerops.com)\n[<img src="https://user-images.githubusercontent.com/5151193/102297674-e6d7ec80-3f0c-11eb-982f-cc5d13b0e9ce.jpg" width="250" hspace="20"/>](https://www.guidepointsecurity.com/)\n[<img src="https://user-images.githubusercontent.com/5151193/95542303-a27f1c00-09b2-11eb-8682-e10b3e0f0710.jpg" width="200" hspace="20"/>](https://lostrabbitlabs.com/)\n\n# Table of Contents\n- [DeathStar](#deathstar)\n  * [Motivation](#motivation)\n  * [New Features](#new-features)\n  * [Official Discord Channel](#official-discord-channel)\n  * [Installation](#installation)\n    + [Docker](#docker)\n    + [Python Package](#python-package)\n    + [Development Install](#development-install)\n  * [Usage](#usage)\n  * [Extending Functionality](#extending-functionality)\n    + [Kyber Crystals](#kyber-crystal-plugin-system)\n    + [Creating Kyber Crystals](#creating-kyber-crystals)\n    + [Crystal Injection](#crystal-injection)\n  * [Defense & Detection](#defense--detection)\n  * [Feature Roadmap & Interest Check](#feature-roadmap--interest-check)\n\n## Motivation\n\nThe primary motivation behind the creation of this was to demonstrate how a lot of the commonly exploited Active Directory misconfiguration can be chained together to gain Administrator level privileges in an automated fashion (akin to a worm).\n\nWhile there are definitely a lot more things that could be taken advantage of (including server side vulnerabilities such as MS17-010), DeathStar mainly focuses on exploiting misconfigurations/vulnerabilities which have a very low probability of causing any sort of system/network stability issues.\n\nVersion 0.2.0 is a complete re-write of the original, implements a [plugin system](#kyber-crystals) (among lots of [other things](#new-features)) which allows anyone to extend it\'s functionality if so desired. \n\nAdditionally, it now supports Active Directory environments with multiple Forests/Domains and has an "Active Monitoring" feature which allows it to adapt it\'s attack path based on real-time changes in the network.\n\n## New Features\n\nVersion 0.2.0 is a complete re-write of the original DeathStar script which I released in 2017.\n\nHere\'s a complete list of the new and shiny things:\n\n- Completely Asynchronous (uses AsyncIO)\n- Has the ability to get Domain Admin & Enterprise Admin rights (as supposed to just Domain Admin rights)\n- Supports environments with Multiple Active Directory Domains & Forests\n- Implements the Kyber Crystal Plugin system that allows anybody to extend it\'s functionality.\n- Active Monitoring: this allows DeathStar to poll all compromised machines for new logins and adapt attack paths accordingly.\n- Uses the [BC-Security Empire Fork](https://github.com/BC-SECURITY/Empire)\n\n## Official Discord Channel\n\nCome hang out on the Porchetta Industries Discord server!\n\n[![Porchetta Industries](https://discordapp.com/api/guilds/736724457258745996/widget.png?style=banner3)](https://discord.gg/ycGXUxy)\n\n## Installation\n\nThe author recommends using Docker or PipX.\n\n### Docker\n\n```\ndocker run --rm -it byt3bl33d3r/deathstar -u <empire_username> -p <empire_password> --api-host <empire_ip>\n```\n\nSince Empire has a Docker image, you could totally write Docker Compose file to get both up and running instantly :)\n\n### Python Package\n\n**This project is available on Pypi under the name `deathstar-empire` because someone else aleady has a project called deathstar**\n\n```\npython3 -m pip install --user pipx\npipx install deathstar-empire\n```\n\n### Development Install\n\nYou should only be installing DeathStar this way if you intend to hack on the source code. You\'re going to Python 3.8+ and [Poetry](https://python-poetry.org/). Please refer to the Poetry installation documentation in order to install it.\n\n```console\ngit clone https://github.com/byt3bl33d3r/DeathStar && cd WitnessMe\npoetry install\n```\n\n## Usage\n\nFirst, you\'re going to need to install [BC-Security\'s Empire fork](ttps://github.com/BC-SECURITY/Empire), please refer to it\'s documentation on how to get it installed.\n\nYou then need to start Empire with it\'s RESTful API enabled, you should also specify a username and password as this is needed for DeathStar to interact with it.\n\n```\npython empire --rest --username <empire_username> --password <empire_password>\n```\n\nPoint DeathStar to Empire and give it the same credentials you specified before. By default, it\'ll attempt to find Empire\'s RESTful API on the loopback interface, you can override this using the ```--api-host``` flag.\n\n```\ndeathstar -u <empire_username> -p <empire_password> --api-host <empire_ip>\n```\n\nDeathStar will login to Empire\'s API and automatically start a listener for you. Now all you have to do is get an Empire agent on a box! DeathStar will immediately take over and do it\'s thang.\n\n## Extending Functionality\n\nThe DeathStar is powered by Kyber Crystals! The Kyber Crystal Plugin System allows anyone to extend DeathStar\'s functionality and enable it to use any of Empire\'s available modules.\n\nAfter creating a Kyber crystal, you have to inject the crystal into Deathstar\'s reactor using a process called Crystal Injection (this is currently a manual process, see the [appropriate section](#crystal-injection)).\n\n## Kyber Crystal Plugin System\n\nKyber Crystals serve as an abstraction layer between Empire\'s module output and DeathStar\'s internal logic. In practical terms, they\'re responsible for initiating the HTTP API calls to run an Empire module, parse their output and return structured data (JSON) that DeathStar\'s internal logic can use.\n\n## Creating Kyber Crystals\n\nA Kyber Crystal is made up of a single Python file which defines an entry coroutine (asynchronous function) named `crystallize`.\n\nThis function **must** be coroutine and have the ```async``` keyword. Additionally, it\'s first argument must always be the Empire Agent Object to run the module on.\n\nKyber Crystals are automatically loaded on runtime from the `crystals` folder and are organized by the task performed and the empire module they run & parse.\n\nSince an example is worth a thousand words, below is the Kyber Crystal code responsible for running the `get_domain_controller` Empire module and can be found in the `deathstar/crystals/recon/get_domain_controller.py` file:\n\n```python\nfrom deathstar.utils import posh_object_parser, beautify_json\n\n\nasync def crystallize(agent):\n    output = await agent.execute(\n        "powershell/situational_awareness/network/powerview/get_domain_controller"\n    )\n\n    results = output["results"]\n    parsed_obj = posh_object_parser(results)\n    log.debug(beautify_json(parsed_obj))\n    return parsed_obj\n```\n\nAs of writing, Empire modules output non-structured data in the form of "stringified" PowerShell Objects or Tables. This is where the `posh_object_parser` and `post_table_parser` functions come in and take care of turning those PoSH objects/tables into JSON (the code for these functions is awful, don\'t look at it). \n\nThese functions don\'t account for every edge case, so if you see some values missing in the returned JSON you\'re probably going to have to parse the module output manually.\n\nThe end result is the coroutine returning the JSON output to be parsed by DeathStar\'s internal logic.\n\nThe `log` variable is injected at runtime into each plugin for debugging purposes. You can enable debug output by passing the ```--debug``` flag to DeathStar.\n\n## Crystal Injection\n\nOnce you created your Kyber Crystal you need to inject it into the reactor so it can be part of the "pwning process". As of writing this has to be done a manually and involves modifying the code in the main `deathstar.py` file.\n\nIn the `deathstar.py` file there\'s a `DeathStar` class which has 4 async methods (coroutines), which are responsible for most of the internal logic. Depending on what your Kyber Crystal does, you\'re probably going to want to run it within one of these 4 coroutines:\n\n- `planetary_recon`: Recon coroutine responsible for performing the initial domain information gathering\n- `launch_hunter_killers`: Domain Privesc coroutine, executes the domain privesc stuff\n- `galaxy_conquest`: Lateral movement coroutine\n- `fire_mission`: Active Monitoring coroutine, this polls each agent for new logins, performs process injection and prioritizes high-value targets (e.g. machines with admins logged in)\n\nCould have I made this easier? Yes, but it would have required a ton more work and honestly I don\'t have any clue if people are even interested in this. See the [Feature Roadmap & Interest Check]() section.\n\n## Defense & Detection\n\nDeathStar merely automates Empire, so to detect DeathStar you need to detect Empire. Consequentially your standard "Offensive PowerShell Tradecraft" defenses and detections apply.\n\nGenerally speaking, this involves making sure you have an EDR solution with AMSI integration, have PowerShell Logging enabled (ScriptBlock, Module and Transcription level logging) and if possible enable PowerShell Constrained Language Mode. While none of these defenses are "bullet-proof" they significantly raise the bar and make life harder for attackers. Defense in depth is the key here.\n\n**DeathStar, by default, does not enable any of the AMSI/Logging bypasses that Empire has and doesn\'t enable any setting(s) that could aid in evading any of the modern PowerShell defenses.**\n\nIf someone using DeathStar doesn\'t know what they\'re doing, they\'ll get caught the second they launch the initial Empire Agent on any modern Windows 10 system. \n\n## Feature Roadmap & Interest Check\n\nThere\'s a lot that could be done with DeathStar, the dream would be to have a tool which can aid in Threat Hunting. It can definitely still be used for Offensive purposes during pentests but I\'m seeing the future of this being a "Purple Team" tool.\n\nSome of the main features I\'d love would be:\n\n- Configuration file to allow tweaking some of Empire\'s settings regarding evasion.\n- Have YAML "Playbooks" which would allow users to select which TTPs/Empire Modules to use during the "pwning" process.\n- Make an easier way to add Kyber Crystals to the main logic (without requiring code modifications)\n \n These would all require a ton more work, and I\'m not even sure if people find this useful as it is. If you like the idea and think it could be useful, feel free to ping me on [Twitter](https://twitter.com/home), [sponsor me on Github](https://github.com/sponsors/byt3bl33d3r/) or send a PR.\n',
    'author': 'Marcello Salvati',
    'author_email': 'byt3bl33d3r@pm.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/byt3bl33d3r/DeathStar',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
